
# q-expansion of newform 63.5.m.b, downloaded from the LMFDB on 27 April 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 63
weight = 5
poly_data = [1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[29, [1, 0]], [10, [0, 1]]]
aps_data = [[0, 0], [0, 0], [0, 0], [-16, -39], [0, 0], [161, -322], [0, 0], [462, -231], [0, 0], [0, 0], [455, 455], [0, 2591], [0, 0], [-23, 0], [0, 0], [0, 0], [0, 0], [8288, -4144], [8809, -8809], [0, 0], [-6111, -6111], [0, 12361], [0, 0], [0, 0], [-224, 448], [0, 0], [24178, -12089], [0, 0], [-18721, 18721], [0, 0], [-20809, 0], [0, 0], [0, 0], [20839, -41678], [0, 0], [36194, -36194], [-19824, -19824], [0, -15506], [0, 0], [0, 0], [0, 0], [-32865, 65730], [0, 0], [-17377, 17377], [0, 0], [21616, 21616], [61486, 0], [-56784, 113568], [0, 0], [-97342, 48671], [0, 0], [0, 0], [64064, 64064], [0, 0], [0, 0], [0, 0], [0, 0], [135520, -67760], [-12191, 12191], [0, 0], [68761, 68761], [0, 0], [-21385, 42770], [0, 0], [-125790, 62895], [0, 0], [0, 217799], [-194063, 0], [0, 0], [-140560, 281120], [0, 0], [0, 0], [-154665, -154665], [0, -54671], [-86039, 0], [0, 0], [0, 0], [331870, -165935], [0, 0], [-123809, -123809], [0, 0], [-123121, 0], [0, 0], [163345, -326690], [94752, -47376], [0, 0], [0, 0], [0, 170591], [0, 0], [-271129, 0], [0, 0], [0, 0], [-352537, 352537], [0, 0], [0, 226199], [0, 0], [0, 0], [0, 0], [-213010, 106505], [0, 527281], [-342382, 0], [0, 0], [0, 0], [0, 0], [619321, -619321], [47215, 47215], [0, 0], [0, 0], [0, 0], [58751, -117502], [348978, -174489], [-516338, 516338], [0, 0], [-284935, -284935], [-342046, 0], [0, 0], [-251111, 502222], [0, 0], [0, 0], [0, 0], [475839, 475839], [905329, 0], [0, 0], [0, 0], [-1098482, 549241], [0, 0], [0, -133006], [0, 0], [-603239, 1206478], [-614978, 307489], [-679319, 679319], [0, 0], [0, -1095961], [443854, 0], [0, 0], [634305, -1268610], [0, 0], [-174944, -174944], [0, 0], [0, 0], [508704, -1017408], [0, 0], [235294, -235294], [0, 0], [371679, 371679], [0, 0], [-454545, 909090], [0, 0], [-1588160, 794080], [0, 0], [0, 1180654], [0, 0], [1363129, 0], [0, 0], [1133591, -1133591], [0, 0], [0, 1685447], [0, 0], [658735, -1317470], [0, 0], [0, 0], [0, 0], [1323191, 0], [0, 0], [0, 0], [0, 0], [902087, -902087], [458689, 458689]]
